package guru.animator;

import java.util.*;

public abstract class BaseAnimator implements Animator {
    Set listener;
    long starttime = 0;
    long dur = 0;
    float start = 0;
    float end =0;
    boolean running = false;

    public BaseAnimator( long dur, float start, float end ) {
        listener = new HashSet();

        this.start = start;
        this.end = end;
        this.dur = dur;
    }

    public void start() {
        this.starttime = System.currentTimeMillis();
        this.running = true;
    }


    public float get() {
        if (!running) {
            return ( starttime == 0 ? start : end );
        } else {
            long now = System.currentTimeMillis() - starttime;
            if ( now > dur ) {
                running = false;
                for( Iterator i=listener.iterator(); i.hasNext(); ) {
                    AnimationListener al = (AnimationListener)i.next();
                    al.animationDone( this );
                }
                return end;
            } else {
                float d = (float)now/(float)dur;
                return f( d, start, end );
            }
        }
    }

    abstract public float f( float d, float start, float stop );

    public boolean isRunning() {
        return running;
    }

    public void reset() {
        this.starttime = 0;
        this.running = false;
    }

    public boolean isDone() {
        return ( !running && starttime > 0 );
    }

    public void addAnimationListener( AnimationListener al ) {
        listener.add( al );
    }
}
