package guru.animator;

import java.util.*;

public class Sequence implements Animator, AnimationListener {
    Set listener;
    List animators;
    Animator current;

    public Sequence() {
        animators = new ArrayList();
        listener = new HashSet();
    }

    public Sequence( Animator[] a ) {
        animators = new ArrayList();
        for( int i =0; i < a.length; i++) {
            addAnimator( a[i] );
        }
        listener = new HashSet();
    }

    public void addAnimator( Animator a ) {
        animators.add( a );
        a.addAnimationListener( this );
        if (current == null ) {
            current = (Animator)animators.get(0);
        }
    }      

    public void start() {
        if (current != animators.get(0)) {
            reset();
        }
        
        current.start();
        
    }

    public void reset() {
        for( int i=0; i < animators.size(); i++) {
            ((Animator)animators.get(i)).reset();
        }
        current = (Animator)animators.get(0);
    }


    public float get() {
        return current.get();
    }

    public boolean isRunning() {
        return current.isRunning();
    }

    public boolean isDone() {
        return current == animators.get(animators.size()-1) && current.isDone();
    }

    public void animationDone( Animator a ) {
        ListIterator iterator = animators.listIterator();
        Animator tmp = null;
        while( iterator.hasNext()) {
           tmp = (Animator)iterator.next();
           if (tmp == a) break;
        }        
        if (iterator.hasNext())  {
            current = (Animator)iterator.next();
            current.start();
        } else {
            for( Iterator i=listener.iterator(); i.hasNext(); ) {
                AnimationListener al = (AnimationListener)i.next();
                al.animationDone( this );
            }
        }
    }

    public void addAnimationListener( AnimationListener al ) {
        listener.add( al );

    }

    public static Sequence parse( String in ) {
        Sequence res = new Sequence();
        String[] parts = in.split( ",");
        float start = 0;
        for( int i =0; i<parts.length; i++) {
            String tmp = parts[i].trim();
            if ( tmp.startsWith("D")) {
                res.addAnimator( new Delay( Long.parseLong(tmp.substring(1)), start));
            } else if ( tmp.startsWith("S")) {
                start = Float.parseFloat(tmp.substring(1));
            } else if ( tmp.startsWith("L")) {
                String[] val = tmp.substring(1).split("->");
                long dur = Long.parseLong(val[0].trim());
                float end = Float.parseFloat(val[1].trim());
                res.addAnimator( new LinearAnimator( dur, start, end ));
                start = end;
            } else if ( tmp.startsWith("R")) {
                String[] val = tmp.substring(1).split("->");
                long dur = Long.parseLong(val[0].trim());
                float end = Float.parseFloat(val[1].trim());
                res.addAnimator( new RootAnimator( dur, start, end ));
                start = end;
            } else if ( tmp.startsWith("Q")) {
                String[] val = tmp.substring(1).split("->");
                long dur = Long.parseLong(val[0].trim());
                float end = Float.parseFloat(val[1].trim());
                res.addAnimator( new SquareAnimator( dur, start, end ));
                start = end;
            }
        }
        return res;
    }
}
